@@ -57,4 +57,8 @@
 
   (list 7 4))
 
+(define (get-options) #f)
+
+(define (apply-options options) #f)
+
 (set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout droppable?)
