@@ -56,25 +56,12 @@ class PlayerTimer(ui.TimerFeedback):
 class ChessGame(game.ChessGame):
     """
     """
-    # Link back to the main application
-    application    = None
-    
-    # The name of the game
-    name           = None
-
     # The view watching this scene
     view           = None
     
     # The players in the game
     __movePlayer   = None
     __aiPlayers    = None
-    
-    # The file this is saved to
-    fileName       = None
-    needsSaving    = True
-    
-    # Date this game started
-    date           = '????.??.??'
 
     # Mapping between piece names and promotion types
     __promotionMapping = {'queen': chess.board.QUEEN, 'knight': chess.board.KNIGHT, 'bishop': chess.board.BISHOP, 'rook': chess.board.ROOK}
@@ -94,6 +81,10 @@ class ChessGame(game.ChessGame):
         self.name = name
         self.__aiPlayers = []
         
+        self.fileName    = None
+        self.inHistory   = False
+        self.needsSaving = False
+        
         # Call parent constructor
         game.ChessGame.__init__(self)
 
@@ -286,6 +277,27 @@ class ChessGame(game.ChessGame):
         # Remove the game from the UI
         self.application._removeGame(self)
         self.view.controller.close()
+
+    def setNeedsSaving(self, needsSaving):
+        """
+        """
+        # Autosaved games don't need saving
+        if self.inHistory:
+            needsSaving = False
+
+        if self.needsSaving == needsSaving:
+            return
+        self.needsSaving = needsSaving
+        self.view.controller.setNeedsSaving(needsSaving)
+
+    def save(self):
+        """Save this game"""
+        if len(self.getMoves()) < 2:
+            return
+        pgnGame = chess.pgn.PGNGame()
+        self.toPGN(pgnGame)
+        self.application.history.save(pgnGame, self.fileName)
+        self.setNeedsSaving(False)
         
 class UI(ui.UIFeedback):
     """
@@ -349,7 +361,10 @@ class UI(ui.UIFeedback):
             b = None
         else:
             b = (game.black.type, game.black.level)
-        g = self.application.addGame(game.name, game.white.name, w, game.black.name, b)
+        g = self.application.addLocalGame(game.name, game.white.name, w, game.black.name, b)
+        if g is None:
+            return
+        g.inHistory = True
         self.application.logger.addLine('Starting game %s between %s (%s) and %s (%s). (%i moves)' % \
                                         (game.name,
                                          game.white.name, str(game.white.type),
@@ -450,17 +465,24 @@ class Application:
         if fd is not None:
             self.ioHandlers.pop(fd)
             
-    def setGame(self, g):
+    def addGame(self, name):
         if self.__game is not None:
-            self._save(self.__game)
-        self.__game = g
-            
-    def addNetworkGame(self, name):
-        g = ChessGame(self, name)
-        self.setGame(g)
-        return g
+            # Save the current game to the history
+            if self.__game.inHistory:
+                response = ui.SAVE_YES
+            elif self.__game.needsSaving:
+                response = self.ui.controller.requestSave('Save current game?')
+            else:
+                response = ui.SAVE_NO
 
-    def addGame(self, name, whiteName, whiteType, blackName, blackType):
+            if response is ui.SAVE_YES:
+                self.__game.save()
+            elif response is ui.SAVE_ABORT:
+                return None
+        self.__game = ChessGame(self, name)
+        return self.__game
+
+    def addLocalGame(self, name, whiteName, whiteType, blackName, blackType):
         """Add a chess game into glChess.
         
         'name' is the name of the game (string).
@@ -474,8 +496,9 @@ class Application:
         # FIXME: Replace arguments with player objects
         
         # Create the game
-        g = ChessGame(self, name)
-        self.setGame(g)
+        g = self.addGame(name)
+        if g is None:
+            return None
 
         msg = ''
         if whiteType is None:
@@ -541,7 +564,9 @@ class Application:
             self.ui.controller.reportGameLoaded(gameProperties)
             return
 
-        newGame = self.addGame(gameProperties.name, gameProperties.white.name, w, gameProperties.black.name, b)
+        newGame = self.addLocalGame(gameProperties.name, gameProperties.white.name, w, gameProperties.black.name, b)
+        if newGame is None:
+            return None
         newGame.date = pgnGame.getTag(chess.pgn.TAG_DATE)
         newGame.fileName = path
         if gameProperties.moves:
@@ -589,9 +614,9 @@ class Application:
         except ValuError:
             blackTime = duration
         newGame.setTimer(duration, whiteTime / 1000, blackTime / 1000)
-
-        # No change from when loaded
-        newGame.needsSaving = False
+        
+        # No need to save freshly loaded game
+        newGame.setNeedsSaving(False)
 
         return newGame
 
@@ -621,8 +646,9 @@ class Application:
         if nArgs == 1:
             self.__autoload()
         
-        # Load requested games
-        for path in sys.argv[1:]:
+        # Load requested game
+        elif nArgs == 2:
+            path = sys.argv[1]
             import time
             self.logger.addLine('loading...')
             s = time.time()
@@ -639,6 +665,18 @@ class Application:
                     g = self.addPGNGame(p[0], path)
             self.logger.addLine('loaded in %f seconds' % (time.time() - s))
 
+        else:
+            print _('Usage: %s [game]') % sys.argv[0]
+            sys.exit(0)
+
+        # Start default game if no game present
+        if self.__game is None and len(profiles) > 0:
+            black = (profiles[0].name, 'easy')
+            name = _('Human versus %s') % profiles[0].name
+            g = self.addLocalGame(name, _('White'), None, _('Black'), black)
+            g.inHistory = True
+            g.start()
+
         # Start UI (does not return)
         try:
             self.ui.controller.run()
@@ -656,33 +694,37 @@ class Application:
 
     def quit(self):
         """Quit glChess"""
-        # Notify the UI
-        self.ui.controller.close()
-
         if self.__game is not None:
-            # Save the current game to the history
-            self._save(self.__game)
-        
+            if self.__game.inHistory:
+                response = ui.SAVE_YES
+            elif self.__game.needsSaving:
+                response = self.ui.controller.requestSave(_('Save game before closing?'))
+            else:
+                response = ui.SAVE_NO
+
+            if response == ui.SAVE_YES:
+                self.__game.save()
+            elif response == ui.SAVE_ABORT:
+                return
+
             # Abort current game (will delete AIs etc)
             self.__game.abort()
 
+        # Notify the UI
+        self.ui.controller.close()
+
         # Exit the application
         sys.exit()
         
     # Private methods
-    
-    def _save(self, g):
-        if len(g.getMoves()) < 2:
-            return
-        pgnGame = chess.pgn.PGNGame()
-        g.toPGN(pgnGame)
-        self.history.save(pgnGame, g.fileName)
 
     def __autoload(self):
         """Restore games from the autosave file"""
         (pgnGame, fileName) = self.history.getUnfinishedGame()
         if pgnGame is not None:
-            self.addPGNGame(pgnGame, fileName)
+            g = self.addPGNGame(pgnGame, fileName)
+            if g is not None:
+                g.inHistory = True
 
 if __name__ == '__main__':
     app = Application()
