@@ -30,7 +30,10 @@
 /**********************************************************************/
 /* File Static Variables                                              */
 /**********************************************************************/
-static gint control_keys[12];
+#define N_KEYS 12
+
+static guint control_keys[N_KEYS];
+
 /**********************************************************************/
 
 
@@ -38,26 +41,6 @@ static gint control_keys[12];
 /* Function Definitions                                               */
 /**********************************************************************/
 
-/**
- * keyboard_string
- * @ksym: KeySym
- *
- * Description:
- * Returns the text description of a keysym
- *
- * Returns:
- * String representation of a keysym
- **/
-gchar *
-keyboard_string (gint ksym)
-{
-  gchar *name;
-
-  name = gdk_keyval_name (ksym);
-
-  return name;
-}
-
 /**
  * keyboard_set
  * @keys: array of keysyms
@@ -66,7 +49,7 @@ keyboard_string (gint ksym)
  * sets the keybaord mapping
  **/
 void
-keyboard_set (gint * keys)
+keyboard_set (guint * keys)
 {
   gint i;
 
@@ -100,7 +83,7 @@ keyboard_cb (GtkWidget * widget, GdkEventKey * event, gpointer data)
    * if N is used as a key, then Ctrl-N is never picked up. The cleaner
    * option, making the signal a connect_after signal skims the arrow keys
    * before we can get to them which is a bigger problem. */
-  if (event->state &= (GDK_CONTROL_MASK | GDK_MOD1_MASK))
+  if (event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK))
     return FALSE;
 
   for (i = 0; i < 12; ++i) {
